import { useContext, useState, createElement, createContext, useReducer, useEffect } from 'react';
import { useDrag, useDrop, DndProvider } from 'react-dnd';
import { HTML5Backend } from 'react-dnd-html5-backend';

function _extends() {
  _extends = Object.assign || function (target) {
    for (var i = 1; i < arguments.length; i++) {
      var source = arguments[i];

      for (var key in source) {
        if (Object.prototype.hasOwnProperty.call(source, key)) {
          target[key] = source[key];
        }
      }
    }

    return target;
  };

  return _extends.apply(this, arguments);
}

var title = {
  fontFamily: 'inherit',
  color: 'inherit',
  fontSize: '16px',
  margin: '0 0 15px 0'
};
var list = {
  display: 'inline-block',
  verticalAlign: 'top',
  margin: '8px',
  padding: '10px 5px 15px 5px',
  minWidth: '200px',
  minHeight: '250px',
  overflow: 'auto',
  boxShadow: '0px 0px 1px rgba(51, 51, 51, 0.25)',
  borderRadius: '5px',
  backgroundColor: '#eeeeee'
};
var card = {
  width: '100%',
  height: 'auto'
};
var style = {
  title: title,
  list: list,
  card: card
};

var Card = function Card(props) {
  var boardContext = useContext(BoardContext);

  var _React$useState = useState('5px'),
      bottomMargin = _React$useState[0],
      setBottomMargin = _React$useState[1];

  var _useDrag = useDrag({
    item: {
      id: props.id,
      value: props.render,
      type: 'card'
    },
    collect: function collect(monitor) {
      return {
        isDragging: monitor.isDragging()
      };
    }
  }),
      dragProps = _useDrag[0],
      drag = _useDrag[1];

  var _useDrop = useDrop({
    accept: ['card'],
    hover: function hover(_item, monitor) {
      boardContext === null || boardContext === void 0 ? void 0 : boardContext.dispatch({
        type: 'hover',
        payload: {
          dataId: props.id,
          listId: props.listId
        }
      });
      setBottomMargin('30px');
      setTimeout(function () {
        if (!monitor.isOver()) setBottomMargin('5px');
      }, 100);
    }
  }),
      drop = _useDrop[1];

  return createElement("div", {
    ref: drag,
    style: _extends({
      display: dragProps.isDragging ? 'none' : 'block',
      marginBottom: bottomMargin
    }, style.card)
  }, createElement("div", {
    ref: drop,
    style: {
      width: '100%',
      height: '100%'
    }
  }, props.children));
};

var List = function List(props) {
  var boardContext = useContext(BoardContext);

  var _useDrop = useDrop({
    accept: ['card'],
    drop: function drop(_item, monitor) {
      var receivedItem = monitor.getItem();
      boardContext === null || boardContext === void 0 ? void 0 : boardContext.dispatch({
        type: 'drop',
        payload: {
          oldListId: receivedItem.listId,
          newListId: props.id,
          dataId: receivedItem.id
        }
      });
    }
  }),
      drop = _useDrop[1];

  return createElement("div", {
    ref: drop,
    style: style.list
  }, createElement("h1", {
    style: style.title
  }, props.title), props.items.map(function (value) {
    return createElement(Card, {
      id: value.dataId,
      listId: value.listId,
      key: value.dataId,
      render: value.render
    }, value.render);
  }));
};

var BoardContext = createContext(null);

function reducer(state, action) {
  switch (action.type) {
    case 'hover':
      return _extends({}, state, {
        hoverItem: action.payload
      });

    case 'drop':
      var selectedItemIndex = state.data.findIndex(function (val) {
        return val.dataId === action.payload.dataId;
      });
      var selectedItem = state.data[selectedItemIndex];
      selectedItem.listId = action.payload.newListId;
      state.data.splice(selectedItemIndex, 1);
      var hoverItemIdex = state.data.findIndex(function (val) {
        return val.dataId === state.hoverItem.dataId;
      });
      var index = hoverItemIdex + 1;

      if (state.hoverItem.listId === action.payload.newListId) {
        state.data.splice(index, 0, selectedItem);
      } else {
        state.data.push(selectedItem);
      }

      state.onDataUpdate(state.data);
      return _extends({}, state, {
        data: [].concat(state.data),
        hoverItem: {
          listId: '',
          dataId: ''
        }
      });

    case 'update':
      return _extends({}, state, {
        data: action.payload
      });

    default:
      return state;
  }
}

var Board = function Board(props) {
  var _React$useReducer = useReducer(reducer, {
    data: props.dataSource,
    onDataUpdate: props.onDataUpdate,
    hoverItem: {
      listId: '',
      dataId: ''
    }
  }),
      state = _React$useReducer[0],
      dispatch = _React$useReducer[1];

  useEffect(function () {
    dispatch({
      type: 'update',
      payload: props.dataSource
    });
  }, [props.dataSource]);
  return createElement(DndProvider, {
    backend: HTML5Backend
  }, createElement(BoardContext.Provider, {
    value: {
      state: state,
      dispatch: dispatch
    }
  }, createElement("div", {
    style: props.style,
    className: props.className
  }, props.lists.map(function (list) {
    return createElement(List, {
      key: list.listId,
      id: list.listId,
      title: list.title,
      items: state.data.filter(function (data) {
        return data.listId === list.listId;
      })
    });
  }))));
};

var Item = function Item(props) {
  var style = {
    display: 'flex',
    alignItems: 'center',
    justifyContent: 'center',
    backgroundColor: '#ffffff',
    height: '30px',
    borderRadius: '2px',
    boxShadow: '0 0 1px rgba(51, 51, 51, 0.25)'
  };
  return createElement("div", {
    style: style
  }, props.children);
};

export { Board, Item };
//# sourceMappingURL=index.modern.js.map
